from abc import ABCMeta, abstractmethod
from pathlib import Path
from typing import NamedTuple

import requests
import requests_cache
from decouple import config
from rich.console import Console
from rich.table import Table

ONE_DAY_IN_SECONDS = 24 * 60 * 60
TIMEOUT = 5

STYLE_HEADER = "dark_orange italic"

console = Console()
error_console = Console(stderr=True, style="bold red")

HOME_DIR = str(Path.home())
CACHE_DB_LOCATION = config("CACHE_DB_LOCATION", default=HOME_DIR)
CACHE_DB_PATH = Path(CACHE_DB_LOCATION) / ".pybites_search_cache.sqlite"
CACHE_EXPIRATION_SECONDS = config(
    "CACHE_EXPIRATION_SECONDS", default=ONE_DAY_IN_SECONDS
)

requests_cache.install_cache(CACHE_DB_PATH, expire_after=CACHE_EXPIRATION_SECONDS)


class ContentPiece(NamedTuple):
    title: str
    url: str
    channel: str


class PybitesSearch(metaclass=ABCMeta):
    @abstractmethod
    def match_content(self, search: str) -> list[ContentPiece]:
        """Search through Pybites content, implement for a specific source"""

    def get_data(self, endpoint):
        return requests.get(endpoint, timeout=TIMEOUT).json()

    def show_matches(self, content: list[ContentPiece]) -> None:
        """Show search results in a nice table"""
        if content:
            table = Table(
                "Title",
                "Url",
                title=self.title,
                header_style=STYLE_HEADER,
                title_style=STYLE_HEADER,
            )
            for row in content:
                table.add_row(row.title, row.url)
            console.print(table)
        else:
            error_console.print("No results found")



import typer

from .all_content import AllSearch
from .article import ArticleSearch
from .bite import BiteSearch
from .podcast import PodcastSearch
from .tip import TipSearch
from .youtube import YouTubeSearch

app = typer.Typer()


@app.command()
def article(search: str):
    searcher = ArticleSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def bite(search: str):
    searcher = BiteSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def podcast(search: str):
    searcher = PodcastSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def tip(search: str):
    searcher = TipSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def video(search: str):
    searcher = YouTubeSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)


@app.command()
def all(search: str):
    searcher = AllSearch()
    results = searcher.match_content(search)
    searcher.show_matches(results)



from rich.table import Table

from .article import ArticleSearch
from .base import STYLE_HEADER, ContentPiece, PybitesSearch, console, error_console
from .bite import BiteSearch
from .podcast import PodcastSearch
from .tip import TipSearch
from .youtube import YouTubeSearch


class AllSearch(PybitesSearch):
    def __init__(self) -> None:
        self.title = "Pybites All Content"

    def match_content(self, search: str) -> list[ContentPiece]:
        classes = (ArticleSearch, BiteSearch, PodcastSearch, TipSearch, YouTubeSearch)
        results = []
        for cls_ in classes:
            searcher = cls_()
            results.extend(searcher.match_content(search))

        return results

    def show_matches(self, content: list[ContentPiece]) -> None:
        """Show search results in a nice table"""
        if content:
            channel = None
            table = Table(
                "Title",
                "Url",
                title=self.title,
                header_style=STYLE_HEADER,
                show_header=False,
                title_style=STYLE_HEADER,
            )
            for row in content:
                if row.channel != channel:
                    if channel:
                        table.add_section()
                    table.add_row(
                        row.channel,
                        "Url",
                        style=STYLE_HEADER,
                        end_section=True,
                    )
                    channel = row.channel
                table.add_row(row.title, row.url)

            console.print(table)
        else:
            error_console.print("No results found")


if __name__ == "__main__":
    searcher = AllSearch()
    results = searcher.match_content("fastapi")
    searcher.show_matches(results)



from .base import ContentPiece, PybitesSearch

YOUTUBE_ENDPOINT = "https://codechalleng.es/api/videos/"
YOUTUBE_BASE_URL = "https://www.youtube.com/watch?v="


class YouTubeSearch(PybitesSearch):
    def __init__(self) -> None:
        self.title = "Pybites YouTube Videos"

    def match_content(self, search: str) -> list[ContentPiece]:
        entries = self.get_data(YOUTUBE_ENDPOINT)
        results = []
        for entry in entries:
            if search.lower() in (entry["title"] + entry["description"]).lower():
                results.append(
                    ContentPiece(
                        title=entry["title"],
                        url=YOUTUBE_BASE_URL + entry["video_id"],
                        channel=self.title,
                    )
                )
        return results


if __name__ == "__main__":
    searcher = YouTubeSearch()
    results = searcher.match_content("django")
    searcher.show_matches(results)



from .base import ContentPiece, PybitesSearch

TIPS_ENDPOINT = "https://codechalleng.es/api/pytips/"


class TipSearch(PybitesSearch):
    def __init__(self) -> None:
        self.title = "Pybites Python Tips"

    def match_content(self, search: str) -> list[ContentPiece]:
        entries = self.get_data(TIPS_ENDPOINT)
        results = []
        for entry in entries:
            if search.lower() in (entry["title"] + entry["description"]).lower():
                results.append(
                    ContentPiece(
                        title=entry["title"], url=entry["link"], channel=self.title
                    )
                )
        return results


if __name__ == "__main__":
    searcher = TipSearch()
    results = searcher.match_content("unpacking")
    searcher.show_matches(results)
